//remotecmd.js
//javascript for Remote Command Line


var KEY_TYPE_CHAR = 0;
var KEY_TYPE_SPECIAL = 1;
var KEY_TYPE_UNICODE = 2;

var BROWSER_IE=0;
var BROWSER_FIREFOX=1;


//arrays of keys that need to be sent on key-down & key-up
//IE's is bigger b/c on special keys, IE sends nothing on the keypress, so we have to catch it on down/up
var keydownkeysIE=[8,9,16,17,18,27,33,34,35,36,37,38,39,40,45,46,112,113,114,115,116,117,118,119,120,121,122,123]; //function keys are 112 to 123
var keydownkeysFF=[16, 17, 18]; //shift, ctrl, alt

var bCtrlDown=false;

//taken from dynamic_refresh.js, JWL has identical logic, but i'd rather own it
var m_isFirefox = (window.navigator !== null) ? (window.navigator.userAgent.indexOf("irefox") !== -1) : false;

//not using yet, since it would require a keypress to initialize
var m_quirks = (document.compatMode == "BackCompat") ? true : false;

var requestNum = -1;

var m_isRCLDebugEnabled = true;

//my own jsdebug
function rcldebug(message) {
    if (m_isRCLDebugEnabled) {
        if ((m_isFirefox) && (typeof console !== 'undefined')) {
            console.debug(message);
        } 
    }
}


//set up keyhandlers to grab keystrokes 
function setKeyHandlers(toggleOn){
	if (toggleOn==true){
		document.onkeydown=doKeydown;
		document.onkeypress=doKeypress;
		document.onkeyup=doKeyup;
	}
	else{
		document.onkeydown=null;
		document.onkeypress=null;
		document.onkeyup=null;	
	}
}

setKeyHandlers(true);

//only called on IE if its not a special character
function doKeypress(event) {
    var lastKeyChar;
    var lastKeyCode;

    var keyType=KEY_TYPE_CHAR;
    if (window.event) {
        // IE
        event = window.event;
        lastKeyCode = event.keyCode;

		//special case for Enter on IE
		if(lastKeyCode==13) {
			keyType=KEY_TYPE_SPECIAL;
		}
    } else if (event.which) {
		//if no character code, send as special
		if(!event.charCode){
			keyType=KEY_TYPE_SPECIAL;
		}
        // Netscape/Firefox/Opera
        if (bCtrlDown){
			lastKeyCode=event.which;
			if (lastKeyCode>=97 && lastKeyCode<=122) 
				lastKeyCode=lastKeyCode-32; //needs to be the uppercase!
			lastKeyCode=lastKeyCode & 63; 
//			alert(lastKeyCode);
		}
		else lastKeyCode = event.which;
		
    }
    else{
        //also mark this as SPECIAL
        lastKeyCode=event.keyCode;  
        keyType=KEY_TYPE_SPECIAL;      
    }
    
    sendKeyEvent(keyType, event, lastKeyCode, "true");

    if (event.preventDefault) {
        event.preventDefault();           
    }
    else event.returnValue=false; //IE
}

//Only going to care about keydown on special keys 
function doKeydown(event) {
    var lastKeyCode;

	//If IE, and its in the array, then just send from here. keyPress won't be called
    if (window.event) {
        // IE
		event = window.event;
        lastKeyCode = event.keyCode;
		keyCheck(BROWSER_IE, event, lastKeyCode, "true");
	}
	else{
		lastKeyCode=event.which;
		rcldebug("doKeydown: "+lastKeyCode);
		if (lastKeyCode==17) 
			bCtrlDown=true;
		keyCheck(BROWSER_FIREFOX, event, lastKeyCode, "true");
	}
}

//Only going to care about keyup on special keys 
function doKeyup(event) {
    var lastKeyCode;

	//If IE, and its in the array, then just send from here. keyPress won't be called
    if (window.event) {
        // IE
		event = window.event;
        lastKeyCode = event.keyCode;
		keyCheck(BROWSER_IE, event, lastKeyCode, "false");
	}
	else{
		lastKeyCode=event.which;
		rcldebug("doKeyup: "+lastKeyCode);
		
		if (lastKeyCode==17) 
			bCtrlDown=false;
		keyCheck(BROWSER_FIREFOX, event, lastKeyCode, "false");
	}	
}

function keyCheck(browser, event, lastKeyCode, press){
	var keyType=KEY_TYPE_SPECIAL; //sending all these as special

	if (browser==BROWSER_IE){
		//efficiency for letters and numbers since IE doesn't support indexOf. 
		//Would be nice if we can just use a broad range like >47 and ditch the whole array.
		if ((lastKeyCode>64 && lastKeyCode<91) || (lastKeyCode>47 && lastKeyCode<58))
			return;
		for (var i =0; i<keydownkeysIE.length; i++){
			if (keydownkeysIE[i] == lastKeyCode){
				sendKeyEvent(keyType, event, lastKeyCode, press); 
				if (event.preventDefault) {
					event.preventDefault();           
				}
				else event.returnValue=false; //IE
				break;
			}
		}		
	}
	else{
		if(keydownkeysFF.indexOf(lastKeyCode)>-1){
			sendKeyEvent(keyType, event, lastKeyCode, press); 
			event.preventDefault();           
		}
	}
}

function sendKeyEvent(keyType, event, key, press) {
    var formId= namespace+"rclForm";
	
	
    if (window.XMLHttpRequest) {
        // Mozilla or IE7+
        sender = new XMLHttpRequest();
    }
    else if (window.ActiveXObject) {
        sender = new ActiveXObject("Microsoft.XMLHTTP");
    }

	//if we're uninitialized, get it from the field. Should happen once on each load
	if (requestNum==-1) {
		requestNum=document.getElementById(namespace+"rclForm:lastRequest").innerHTML;
	}

	rcldebug("sending "+key+" requestNum "+requestNum);
	requestNum++;
    sender.open("GET",m_contextPath+"/RemoteCommandLineServlet?inputCharacter="+key+"&panelId="+formId+":mainPanelBox&press="+press+"&keyType="+keyType+"&requestNum="+requestNum ,true);
    sender.send(null);

}
   
function loginPrompt(nodeChange){
	
	sendAck(nodeChange.Id);
	
	if(nodeChange.MSG_TYPE == 0){	
		setKeyHandlers(false);
		document.getElementById(namespace+"rclForm:ajButton").click();
	}
	else if(nodeChange.MSG_TYPE == 1){
		setKeyHandlers(true);
	}
	watchForChange();
	
}
